import unittest

from easylib.nlp.vocabs import VocabGenerator, VocabLengthFilter, VocabEmptyFilter, VocabSpecialTokensFilter
from easylib.nlp.vocabs import VocabSpaceLineSplitor



class VocabsTest(unittest.TestCase):

    def testVocabGenerator(self):
        filters = [
            VocabLengthFilter(5),
            VocabEmptyFilter(),
            VocabSpecialTokensFilter(['\t', '\n']),
        ]
        splitor = VocabSpaceLineSplitor()
        g = VocabGenerator(10, splitor, filters, min_count=1)

        lines = [
            '\t \n hello world \s   \n',
        ]
        output_file = '/tmp/vocab.test.txt'

        g.generate_from_iterable(lines, output_file)


if __name__ == "__main__":
    unittest.main()



import unittest
import logging

from easylib.log import TimedRotatingLoggerInitializer, RotatingLoggerInitializer


class LoggerInitializerTest(unittest.TestCase):

    def testTimedRotatingLoggerInitializer(self):
        logger = logging.getLogger('easylib')
        logger.info('Info from logger easylib.')

        initializer = TimedRotatingLoggerInitializer(name='easylib2', path='/tmp/easylib2.log')
        initializer.initialize()
        logger = logging.getLogger('easylib2')
        logger.info('Info from logger easylib2.')

    def testRotatingLoggerInitializer(self):
        initializer = RotatingLoggerInitializer(name='easylib-rotating-logger', path='/tmp/easylib.rotating.log')
        initializer.initialize()
        logger = logging.getLogger('easylib-rotating-logger')
        logger.info('Info from logger %s' % logger.name)


if __name__ == '__main__':
    unittest.main()



import logging

from logging.handlers import TimedRotatingFileHandler
from logging.handlers import RotatingFileHandler


class TimedRotatingLoggerInitializer:

    def __init__(self,
                 path='/tmp/easylib.log',
                 name=None,
                 when='D',
                 interval=1,
                 backup_count=5,
                 format='%(asctime)s - %(levelname)s - %(name)s - %(filename)s - %(lineno)d - %(funcName)s -%(message)s',
                 level=logging.INFO):
        self.name = name
        self.path = path
        self.level = level
        self.when = when
        self.interval = interval
        self.backup_count = backup_count
        self.format = format

    def initialize(self):
        formatter = logging.Formatter(self.format)
        handler = TimedRotatingFileHandler(
            filename=self.path,
            when=self.when,
            interval=self.interval,
            backupCount=self.backup_count,
            encoding='utf8')
        handler.setFormatter(formatter)
        logger = logging.getLogger(self.name)
        logger.addHandler(handler)
        logger.setLevel(self.level)
        logger.info('Logger %s initialized.' % logger.name)


class RotatingLoggerInitializer:

    def __init__(self,
                 path='/tmp/easylib.log',
                 name=None,
                 max_bytes=5 * 1024 * 1024,
                 backup_count=5,
                 format='%(asctime)s - %(levelname)s - %(name)s - %(filename)s - %(lineno)d - %(funcName)s -%(message)s',
                 level=logging.INFO):
        self.name = name
        self.format = format
        self.backup_count = backup_count
        self.path = path
        self.max_bytes = max_bytes
        self.level = level

    def initialize(self):
        formatter = logging.Formatter(self.format)
        handler = RotatingFileHandler(
            filename=self.path,
            mode='a',
            maxBytes=self.max_bytes,
            backupCount=self.backup_count,
            encoding='utf8')
        handler.setFormatter(formatter)
        logger = logging.getLogger(self.name)
        logger.addHandler(handler)
        logger.setLevel(self.level)
        logger.info('Logger %s initialized.' % logger.name)



import os


class FileLineReadCallback:

    def read_line(self, line):
        raise NotImplementedError()


class FileLineStripCallback(FileLineReadCallback):

    def read_line(self, line):
        if not line:
            return None
        line = line.strip('\n')
        return line


class FileLineReader:
    """Read a file line by line."""

    def __init__(self, callbacks=None, params=None):
        """Init.

        Args:
            callbacks: A list of `FileLineReadCallback` instance or a list of functions, whose signature: fun(line: str) -> Any
            params: A dict, override the default params.
        """
        default_params = self.default_params()
        if params:
            default_params.update(params)
        self.params = default_params

        self.callbacks = callbacks

    def read_file(self, filename):
        if not os.path.exists(filename):
            print('Files does not exist: %s' % filename)
            return
        with open(filename, mode=self.params['mode'], encoding=self.params['encoding'],buffering=self.params['buffering']) as fin:
            for line in fin:
                if self.callbacks:
                    for c in self.callbacks:
                        if isinstance(c, FileLineReadCallback):
                            # instance of FileLineReadCallback
                            c.read_line(line)
                        else:
                            # a function, signature is: fun(line:str) -> Any
                            c(line)

    def read_files(self, files):
        for f in files:
            self.read_file(f)

    def default_params(self):
        p = {
            'mode': 'rt',
            'encoding': 'utf8',
            'buffering': 8192,
        }
        return p



import unittest

from .file_reader import FileLineReader, FileLineStripCallback, FileLineReadCallback


class FileLinePrintCallback(FileLineReadCallback):

    def read_line(self, line):
        print(line)


class FileReaderTest(unittest.TestCase):

    def testFileLineReader(self):
        def split_fn(line):
            return line.split(' ')

        callbacks = [
            FileLineStripCallback(),
            split_fn,
            FileLinePrintCallback(),
        ]
        r = FileLineReader(callbacks=callbacks)

        r.read_file('easylib/io/file_reader.py')


if __name__ == '__main__':
    unittest.main()



import os


class FileSpliter:
    """Split a file to multi parts."""

    def split(self, file, num_parts, output_dir):
        """Split file.

        Args:
            file: The path of file.
            num_parts: The number of parts you want to split
            output_dir: A dir to save the split parts files
        """
        total_lines = self._count_file_lines(file)
        lines_each_part = int(total_lines // num_parts)
        if total_lines % num_parts == 0:
            self._split_file(file, num_parts, lines_each_part, output_dir)
        else:
            self._split_file(file, num_parts + 1, lines_each_part, output_dir)

    def _split_file(self, file, parts, lines_each_part, output_dir):
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        path, name = os.path.split(file)
        with open(file, mode='rt', encoding='utf8', buffering=8192) as fin:
            for i in range(parts):
                p = os.path.join(output_dir, name + '.part' + str(i))
                with open(p, mode='wt', encoding='utf8', buffering=8192) as fout:
                    for j in range(lines_each_part):
                        line = fin.readline()
                        if not line:
                            break
                        fout.write(line)

    def _count_file_lines(self, f):
        with open(f) as f:
            for i, l in enumerate(f):
                pass
        return i + 1



import os
from collections import Counter


class VocabLineSplitor:
    """Vocab line splitor interface."""

    def split(self, line):
        """Split line to a list of vocabs.

        Args:
            line: A python string. Maybe None.

        Returns:
            A list of vocabs.
        """
        raise NotImplementedError()


class VocabSpaceLineSplitor(VocabLineSplitor):
    """Split line by space."""

    def split(self, line):
        if not line:
            return []
        return line.split(' ')


class VocabFilter:
    """Vocab filter interface."""

    def filter(self, vocab):
        """Filter this vocab if return True, else keep it.

        Args:
            vocab: A python string, vocab

        Returns:
            True if remove this vocab, False to keep this vocab.
        """
        raise NotImplementedError()


class VocabLengthFilter(VocabFilter):
    """Filter vocab if it's length greater than max_len limit."""

    def __init__(self, max_len=10):
        self.max_len = max_len

    def filter(self, vocab):
        if not vocab:
            return True
        return len(vocab) > self.max_len


class VocabEmptyFilter(VocabFilter):
    """Filter empty string."""

    def filter(self, vocab):
        if not vocab:
            return True
        return False


class VocabSpecialTokensFilter(VocabFilter):
    """Filter special tokens."""

    def __init__(self, tokens):
        self.tokens = tokens
        
    def filter(self, vocab):
        for t in self.tokens:
            vocab = vocab.replace(t, '')
            if not vocab:
                return True
        if not vocab:
            return True
        return False


class VocabGenerator:

    def __init__(self, vocab_size, line_splitor, filters=None, min_count=5, use_unk=True, sortby='freq_desc'):
        """Init.

        Args:
            vocab_size: A python integer, vocab size. Not include `<unk>`.
            line_splitor: Split a line to a list of vocabs. Instance of `VocabLineSplitor`.
            filters: An iterable, vocab filters, each element is instance of `VocabFilter`
            min_count: A python integer, word's frequency less than this will not be collected.
            use_unk: A python boolean, add `<unk>` to vocab or not.
            sortby: A python string. choices: ['freq_desc', 'alphabet_asc']
        """
        self.vocab_size = vocab_size
        self.line_splitor = line_splitor
        self.filters = filters
        self.min_count = min_count
        self.use_unk = use_unk

        if sortby not in {'freq_desc', 'alphabet_asc'}:
            raise ValueError('Invalid sortby value: %s' % sortby)
        self.sortby = sortby

        self.counter = Counter()

    def generate(self, input_files, output_file):
        """Generate vocab file.

        Args:
            input_files: An iterable, corpus files.
            output_file: The output vocab file.
        """
        for f in input_files:
            if not os.path.exists(f):
                print('File does not exist: %s' % f)
                continue
            with open(f, mode='rt', encoding='utf8', buffering=8192) as fin:
                for line in fin:
                    words = self.line_splitor.split(line.strip('\n'))
                    if not words:
                        continue
                    self.counter.update(words)
            print('Finished to count file: %s' % f)

        self._write_to_file(output_file)

    def generate_from_iterable(self, lines, output_file):
        for line in lines:
            words = self.line_splitor.split(line.strip('\n'))
            if not words:
                continue
            self.counter.update(words)
        self._write_to_file(output_file)
    
    def _write_to_file(self, output_file):
         with open(output_file, mode='wt', encoding='utf8', buffering=8192) as fout:
            vocabs = []
            for k, v in self.counter.most_common(self.vocab_size):
                if v < self.min_count:
                    continue
                if k == '<unk>':
                    continue
                if self.filters and any(vf.filter(k) for vf in self.filters):
                    continue
                vocabs.append(k)

            if self.sortby == 'alphabet_asc':
                vocabs = sorted(vocabs)
            fout.write('<unk>' + '\n')
            for v in vocabs:
                fout.write(v + '\n')





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="easylib",
    version="0.0.4",
    description="Frequently used functions and modules library for Python 3 by ZhouYang Luo.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/luozhouyang/easylib",
    author="ZhouYang Luo",
    author_email="zhouyang.luo@gmail.com",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[

    ],
    classifiers=(
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    )
)



import unittest

from easylib.io.file_spliter import FileSpliter


class FileSpliterTest(unittest.TestCase):

    def testFileSpliter(self):
        s = FileSpliter()
        input_file = 'easylib/io/file_reader.py'
        output_dir = 'testdata'
        s.split(input_file, 2, output_dir)


if __name__ == '__main__':
    unittest.main()
